using System;
using System.Collections.Generic;
using System.Text;
using System.Collections.ObjectModel;

namespace M2xdam.VistaFlickrUploader
{
    class PictureListProvider : ObservableCollection<PictureItem>
    {
        protected override void RemoveItem(int index)
        {
            // Call Dispose to release file handle
            this[index].Dispose();
            base.RemoveItem(index);
        }

        protected override void ClearItems()
        {
            // Call Dispose to release file handle
            foreach (PictureItem picture in this)
            {
                picture.Dispose();
            }
            base.ClearItems();
        }

        protected override void SetItem(int index, PictureItem item)
        {
            // Call Dispose to release file handle
            this[index].Dispose();
            base.SetItem(index, item);
        }

        public PictureItem Find(PictureItem pictureToFind)
        {
            foreach (PictureItem picture in this)
            {
                if (picture.Guid == pictureToFind.Guid)
                {
                    return picture;
                }
            }
            return null;
        }

        public void Remove(Guid pictureGuid)
        {
            foreach (PictureItem picture in this)
            {
                if (picture.Guid == pictureGuid)
                {
                    Remove(picture);
                    return;
                }
            }
            throw new InvalidOperationException("Trying to remove a picture by a guid that does not exist.");
        }
    }
}
