using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace M2xdam.VistaFlickrUploader
{
    /// <summary>
    /// Class that contains status values of the upload thread
    /// </summary>
    internal class UploadThreadStatus
    {
        private bool forFriends;
        public bool ForFriends
        {
            get { return forFriends; }
            set { forFriends = value; }
        }

        private bool forFamily;
        public bool ForFamily
        {
            get { return forFamily; }
            set { forFamily = value; }
        }

        private bool forEveryone;
        public bool ForEveryone
        {
            get { return forEveryone; }
            set { forEveryone = value; }
        }

        private string photoSetId;
        public string PhotoSetId
        {
            get { return photoSetId; }
            set { photoSetId = value; }
        }

        /// <summary>
        /// Threading variable that contains the index of the currently uploading picture.
        /// </summary>
        private int _currentPictureIndex;
        public int CurrentPictureIndex
        {
            get { return _currentPictureIndex; }
            set { _currentPictureIndex = value; }
        }

        /// <summary>
        /// Threading variable that contains a list of pictures that should be uploaded.
        /// </summary>
        private List<PictureItem> _pictures;
        internal List<PictureItem> Pictures
        {
            get { return _pictures; }
            set { _pictures = value; }
        }

        /// <summary>
        /// An exception that occurred during upload.
        /// </summary>
        private Exception _exception;
        public Exception Exception
        {
            get { return _exception; }
            set { _exception = value; }
        }

        /// <summary>
        /// Stopwatch that keeps track of how long the upload takes.
        /// </summary>
        private Stopwatch _stopwatch;

        /// <summary>
        /// Upload is canceled when this variable is set to true.
        /// </summary>
        private bool _cancelUpload;
        public bool CancelUpload
        {
            get { return _cancelUpload; }
            set { _cancelUpload = value; }
        }

        /// <summary>
        /// Percentage the upload of the current picture is complete.
        /// </summary>
        private byte _picturePercentageComplete;
        public byte PicturePercentageComplete
        {
            get { return _picturePercentageComplete; }
            set { _picturePercentageComplete = value; }
        }

        public PictureItem CurrentPicture
        {
            get { return Pictures[CurrentPictureIndex]; }
        }
        public bool HasException
        {
            get { return this.Exception != null; }
        }
        public bool IsPictureComplete
        {
            get { return (PicturePercentageComplete == 100); }
        }
        public bool IsBatchComplete
        {
            get { return (CurrentPictureIndex == Pictures.Count - 1) && IsPictureComplete; }
        }
        public byte BatchPercentageComplete
        {
            get
            {
                if (IsBatchComplete)
                {
                    return 100;
                }
                else
                {
                    return (byte)((100 * CurrentPictureIndex + PicturePercentageComplete) / Pictures.Count);
                }
            }
        }
        public TimeSpan ElapsedTime
        {
            get { return _stopwatch.Elapsed; }
        }

        public UploadThreadStatus()
        {
            _stopwatch = new Stopwatch();
        }

        public void StartStopwatch()
        {
            _stopwatch.Reset();
            _stopwatch.Start();
        }

        public void StopStopwatch()
        {
            _stopwatch.Stop();
        }
    }
}
