using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Threading;
using System.Collections.ObjectModel;
using System.IO;
using System.ComponentModel;

namespace M2xdam.VistaFlickrUploader
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>

    public partial class Window1 : System.Windows.Window
    {
        /// <summary>
        /// The main list that contains all the selected pictures.
        /// </summary>
        private PictureListProvider pictureListProvider;

        /// <summary>
        /// The Flickr helper object.
        /// </summary>
        private FlickrHelper _flickrHelper;

        private delegate void UpdateForUploadCallback(UploadThreadStatus e);
        private delegate void UpdateForPhotoSets(List<FlickrNet.Photoset> photoSets);

        public Window1()
        {
            InitializeComponent();

            pictureListProvider = new PictureListProvider();
            pictureListProvider.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(pictureListProvider_CollectionChanged);

            MediaPanel.ItemsSource = pictureListProvider;

            ConnectToFlickr(false);

            #region Control events
            this.Closing += new System.ComponentModel.CancelEventHandler(Window1_Closing);
            this.LocationChanged += new EventHandler(Window1_LocationChanged);

            MediaPanel.DragOver += new DragEventHandler(MediaPanel_DragOver);
            MediaPanel.Drop += new DragEventHandler(MediaPanel_Drop);
            MediaPanel.KeyDown += new KeyEventHandler(MediaPanel_KeyDown);
            
            btnUpload.Click += new RoutedEventHandler(btnUpload_Click);

            btnAdd.Click += new RoutedEventHandler(btnAdd_Click);

            btnCancelUpload.Click += new RoutedEventHandler(btnCancelUpload_Click);
            btnConnect.Click += new RoutedEventHandler(btnConnect_Click);

            cbPublic.Checked += new RoutedEventHandler(cbPublic_Checked);
            cbPublic.Unchecked += new RoutedEventHandler(cbPublic_Checked);

            cbAddToSet.Checked += new RoutedEventHandler(cbAddToSet_Checked);
            cbAddToSet.Unchecked += new RoutedEventHandler(cbAddToSet_Checked);
            #endregion

            RestoreWindowPosition();

            MainTabControl.SelectedIndex = 0;
            btnNextFromSelection.IsEnabled = false;

            TabSelection.Visibility = Visibility.Collapsed;
            TabUpload.Visibility = Visibility.Collapsed;

            cbFamily.IsChecked  = Properties.Settings.Default.FlickrPermissionFamily;
            cbFriends.IsChecked = Properties.Settings.Default.FlickrPermissionFriends;
            cbPublic.IsChecked  = Properties.Settings.Default.FlickrPermissionPublic;

            pictureListProvider_CollectionChanged(this, null);
            cbAddToSet_Checked(this, null);
        }

        void cbAddToSet_Checked(object sender, RoutedEventArgs e)
        {
            //Visibility visibility = cbAddToSet.IsChecked.Value ? Visibility.Visible : Visibility.Hidden;
            listSets.Visibility  = Visibility.Hidden;
            btnNewSet.Visibility = Visibility.Hidden;

            if (cbAddToSet.IsChecked.Value)
            {
                // Load list of photosets in background
                this.Cursor = Cursors.Wait;
                BackgroundWorker photoSetLoader = new BackgroundWorker();
                photoSetLoader.DoWork += new DoWorkEventHandler(photoSetLoader_DoWork);
                photoSetLoader.RunWorkerAsync();
            }
        }

        /// <summary>
        /// Loads the list of photo sets for the current user.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void photoSetLoader_DoWork(object sender, DoWorkEventArgs e)
        {
            List<FlickrNet.Photoset> photoSets = _flickrHelper.GetPhotoSets();
            Dispatcher.Invoke(DispatcherPriority.Normal, new UpdateForPhotoSets(this.FillPhotoSetList), photoSets);
        }

        /// <summary>
        /// Fills the list of photo sets with the supplied photo sets.
        /// </summary>
        /// <param name="photoSets"></param>
        private void FillPhotoSetList(List<FlickrNet.Photoset> photoSets)
        {
            listSets.ItemsSource = photoSets;
            listSets.Visibility  = Visibility.Visible;
            this.Cursor = Cursors.Arrow;
        }

        void Window1_LocationChanged(object sender, EventArgs e)
        {
            this.LocationChanged -= new EventHandler(Window1_LocationChanged);
            try
            {
                // Snap window to screen borders
                foreach (System.Windows.Forms.Screen screen in System.Windows.Forms.Screen.AllScreens)
                {
                    if (Math.Abs(screen.WorkingArea.Left - this.Left) < 20)
                    {
                        this.Left = screen.WorkingArea.Left;
                    }
                    if (Math.Abs(screen.WorkingArea.Top - this.Top) < 20)
                    {
                        this.Top = screen.WorkingArea.Top;
                    }
                    if (Math.Abs(screen.WorkingArea.Right - (this.Left + this.Width)) < 20)
                    {
                        this.Left = screen.WorkingArea.Right - this.Width;
                    }
                    if (Math.Abs(screen.WorkingArea.Bottom - (this.Top + this.Height)) < 20)
                    {
                        this.Top = screen.WorkingArea.Bottom - this.Height;
                    }
                }
            }
            finally
            {
                this.LocationChanged += new EventHandler(Window1_LocationChanged);
            }
        }

        /// <summary>
        /// Restores the window size and location of the previous run.
        /// </summary>
        private void RestoreWindowPosition()
        {
            if (Properties.Settings.Default.PropertyValues["WindowSize"].PropertyValue != null)
            {
                this.Width  = Properties.Settings.Default.WindowSize.Width;
                this.Height = Properties.Settings.Default.WindowSize.Height;

                if (this.Width > System.Windows.Forms.SystemInformation.VirtualScreen.Width)
                {
                    this.Width = System.Windows.Forms.SystemInformation.VirtualScreen.Width;
                }
                if (this.Height > System.Windows.Forms.SystemInformation.VirtualScreen.Height)
                {
                    this.Height = System.Windows.Forms.SystemInformation.VirtualScreen.Height;
                }
            }

            if (Properties.Settings.Default.PropertyValues["WindowPosition"].PropertyValue != null)
            {
                this.Left = Properties.Settings.Default.WindowPosition.X;
                this.Top  = Properties.Settings.Default.WindowPosition.Y;

                if (this.Left < System.Windows.Forms.SystemInformation.VirtualScreen.Left)
                {
                    this.Left = System.Windows.Forms.SystemInformation.VirtualScreen.Left;
                }
                if (this.Top < System.Windows.Forms.SystemInformation.VirtualScreen.Top)
                {
                    this.Top = System.Windows.Forms.SystemInformation.VirtualScreen.Top;
                }
                if (this.Left > System.Windows.Forms.SystemInformation.VirtualScreen.Right - this.Width)
                {
                    this.Left = System.Windows.Forms.SystemInformation.VirtualScreen.Right - this.Width;
                }
                if (this.Top > System.Windows.Forms.SystemInformation.VirtualScreen.Bottom - this.Height)
                {
                    this.Top = System.Windows.Forms.SystemInformation.VirtualScreen.Bottom - this.Height;
                }
            }
        }

        void Window1_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            Properties.Settings.Default.FlickrPermissionPublic  = cbPublic.IsChecked.Value;
            Properties.Settings.Default.FlickrPermissionFamily  = cbFamily.IsChecked.Value;
            Properties.Settings.Default.FlickrPermissionFriends = cbFriends.IsChecked.Value;
            Properties.Settings.Default.WindowSize = new Size(this.Width, this.Height);
            Properties.Settings.Default.WindowPosition = new Point(this.Left, this.Top);
            Properties.Settings.Default.Save();
        }

        void cbPublic_Checked(object sender, RoutedEventArgs e)
        {
            cbFamily.IsChecked  = cbPublic.IsChecked;
            cbFriends.IsChecked = cbPublic.IsChecked;
            cbFamily.IsEnabled  = !cbPublic.IsChecked.Value;
            cbFriends.IsEnabled = !cbPublic.IsChecked.Value;
        }

        void btnConnect_Click(object sender, RoutedEventArgs e)
        {
            ConnectToFlickr(true);
        }

        /// <summary>
        /// Connects to the Flickr server and displays the username and buddy icon.
        /// </summary>
        private void ConnectToFlickr(bool allowAuthorizationProcedure)
        {
            txtConnectionError.Visibility = Visibility.Collapsed;
            btnConnect.Visibility = Visibility.Collapsed;

            try
            {
                _flickrHelper = new FlickrHelper(allowAuthorizationProcedure);
                _flickrHelper.OnPictureUploadProgress += new FlickrHelper.UploadEventHandler(flickrHelper_OnPictureUploadProgress);
                _flickrHelper.OnUploadException += new FlickrHelper.UploadEventHandler(flickrHelper_OnPictureUploadException);
                _flickrHelper.OnUploadComplete += new FlickrHelper.UploadEventHandler(flickrHelper_OnUploadComplete);
                
                lblUser.Text = _flickrHelper.CurrentUser.RealName;
                if (_flickrHelper.CurrentStatus.IsPro)
                {
                    lblUser.Text += " (Pro)";
                }

                imgBuddyIcon.Source = BitmapFrame.Create(new Uri(_flickrHelper.CurrentUser.BuddyIconUrl));

                //lblUploadStatus.Text = string.Format("After uploading {0}Mb you've uploaded {2}% of your maximum {1}Mb.", _flickrHelper.CurrentStatus.BandwidthUsed, _flickrHelper.CurrentStatus.BandwidthMax >> 20, _flickrHelper.CurrentStatus.PercentageUsed);

                groupFlickrProperties.Visibility = Visibility.Visible;
            }
            catch (Exception ex)
            {
                _flickrHelper = null;

                lblUser.Text = Properties.Messages.LblConnectionFailed;
                txtConnectionError.Text = ex.Message;

                txtConnectionError.Visibility = Visibility.Visible;
                btnConnect.Visibility = Visibility.Visible;

                groupFlickrProperties.Visibility = Visibility.Hidden;
            }

            // Update upload button
            pictureListProvider_CollectionChanged(null, null);
        }

        void btnCancelUpload_Click(object sender, RoutedEventArgs e)
        {
            _flickrHelper.CancelUpload();
        }

        void pictureListProvider_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            // Enabled/disable upload button
            btnUpload.IsEnabled = (_flickrHelper != null) && (pictureListProvider.Count > 0);

            long totalBytes = 0;
            foreach (PictureItem picture in pictureListProvider)
            {
                totalBytes += picture.FileInfo.Length;
            }
            txtSelectionInfo.Text = string.Format("{0} picture{1}, {2} KB", pictureListProvider.Count, pictureListProvider.Count==1?"":"s", totalBytes >> 10);
        }

        void btnAdd_Click(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.OpenFileDialog openFileDialog = new Microsoft.Win32.OpenFileDialog();

            openFileDialog.CheckFileExists = true;
            openFileDialog.CheckPathExists = true;
            openFileDialog.Multiselect = true;

            // Start in the last used picture directory
            if (Properties.Settings.Default.AddPictureInitialDirectory.Length <= 0)
            {
                Properties.Settings.Default.AddPictureInitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.MyPictures);
            }
            openFileDialog.InitialDirectory = Properties.Settings.Default.AddPictureInitialDirectory;

            bool? success = openFileDialog.ShowDialog(this);

            if (success.HasValue && success.Value == true)
            {
                // Load selected pictures
                foreach (string fileName in openFileDialog.FileNames)
                {
                    AddPicture(fileName);
                }

                // Remember location of pictures
                Properties.Settings.Default.AddPictureInitialDirectory = System.IO.Path.GetDirectoryName(openFileDialog.FileName);
                Properties.Settings.Default.Save();
            }
        }

        void MediaPanel_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.Delete:
                    // Remove all selected items
                    while (MediaPanel.SelectedItem != null)
                    {
                        pictureListProvider.Remove((PictureItem)MediaPanel.SelectedItem);
                    }
                    break;
            }
        }

        private void AddPicture(string fileName)
        {
            try
            {
                pictureListProvider.Add(new PictureItem(fileName));
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, string.Format(Properties.Messages.MsgLoadPictureFailed, fileName) + "\r\n" + ex.Message, this.Title, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        void MediaPanel_Drop(object sender, DragEventArgs e)
        {
            string[] fileNames = e.Data.GetData(DataFormats.FileDrop, true) as string[];
            foreach (string fileName in fileNames)
            {
                AddPicture(fileName);
            }

            e.Handled = true;
        }

        void MediaPanel_DragOver(object sender, DragEventArgs e)
        {
            e.Effects = DragDropEffects.Copy;
            e.Handled = true;
        }

        void btnUpload_Click(object sender, RoutedEventArgs e)
        {
            pictureProgressBar.Value = 0;
            batchProgressBar.Value   = 0;
            lblUploadProgress.Text   = "";
            TabUpload.IsSelected = true;
            
            try
            {
                string photoSetId = cbAddToSet.IsChecked.Value ? ((FlickrNet.Photoset)listSets.SelectedItem).PhotosetId  : null;

                _flickrHelper.UploadPicturesAsync(pictureListProvider,
                                            cbFamily.IsChecked.Value, 
                                            cbFriends.IsChecked.Value, 
                                            cbPublic.IsChecked.Value, 
                                            photoSetId);
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, this.Title, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        void flickrHelper_OnUploadComplete(object sender, UploadThreadStatus e)
        {
            Dispatcher.Invoke(DispatcherPriority.Normal, new UpdateForUploadCallback(this.UpdateForUploadComplete), e);
        }

        void flickrHelper_OnPictureUploadException(object sender, UploadThreadStatus e)
        {
            Dispatcher.Invoke(DispatcherPriority.Normal, new UpdateForUploadCallback(this.UpdateForUploadException), e);
        }

        void flickrHelper_OnPictureUploadProgress(object sender, UploadThreadStatus e)
        {
            Dispatcher.Invoke(DispatcherPriority.Normal, new UpdateForUploadCallback(this.UpdateForPictureUploadProgress), e);
        }

        private void UpdateForUploadException(UploadThreadStatus e)
        {
            string message = string.Format(Properties.Messages.MsgUploadFailed, e.CurrentPictureIndex + 1, e.Pictures.Count, e.CurrentPicture.FileInfo.Name);
            MessageBox.Show(this, message + " " + e.Exception.Message, this.Title, MessageBoxButton.OK, MessageBoxImage.Error);
        }

        private void UpdateForUploadComplete(UploadThreadStatus e)
        {
            if (!e.HasException)
            {
                string message = string.Format(Properties.Messages.MsgSuccessfulUpload, e.Pictures.Count, e.ElapsedTime.TotalSeconds);
                MessageBox.Show(this, message, this.Title, MessageBoxButton.OK, MessageBoxImage.Information);
                pictureListProvider.Clear();
            }

            // Return to the selection tab
            MainTabControl.SelectedIndex = 0;
        }

        string _fileNameCurrentUpload = null;
        private void UpdateForPictureUploadProgress(UploadThreadStatus e)
        {
            // Update progress bars
            batchProgressBar.Value   = e.BatchPercentageComplete;
            pictureProgressBar.Value = e.PicturePercentageComplete;

            lblUploadProgress.Text = string.Format(Properties.Messages.LblUploadProgress, e.CurrentPictureIndex + 1, e.Pictures.Count, e.CurrentPicture.FileInfo.Name);

            // Update picture preview
            if (_fileNameCurrentUpload != e.CurrentPicture.FileInfo.FullName)
            {
                imgCurrentUpload.Source = BitmapFrame.Create(new Uri(e.CurrentPicture.FileInfo.FullName), BitmapCreateOptions.None, BitmapCacheOption.None);
                _fileNameCurrentUpload  = e.CurrentPicture.FileInfo.FullName;
            }

            // Remove uploaded picture from list
            if (e.IsPictureComplete)
            {
                pictureListProvider.Remove(e.CurrentPicture.Guid);
            }
        }
    }
}